@@ -14,13 +14,11 @@ Sub::Exporter - a sophisticated exporter for custom-built routines
 
 =head1 VERSION
 
-version 0.975
-
-  $Id: /my/cs/projects/Sub-Exporter/trunk/lib/Sub/Exporter.pm 31990 2007-07-06T02:33:04.864653Z rjbs  $
+version 0.978
 
 =cut
 
-our $VERSION = '0.975';
+our $VERSION = '0.978';
 
 =head1 SYNOPSIS
 
@@ -178,9 +176,29 @@ The following keys are valid in C<%config>:
   groups  - a list of groups to provide for exporting; each must be followed by
             either (a) a list of exports, possibly with arguments for each
             export, or (b) a generator
+
   collectors - a list of names into which values are collected for use in
                routine generation; each name may be followed by a validator
 
+In addition to the basic options above, a few more advanced options may be
+passed:
+
+  into_level - how far up the caller stack to look for a target (default 0)
+  into       - an explicit target (package) into which to export routines
+
+In other words: Sub::Exporter installs a C<import> routine which, when called,
+exports routines to the calling namespace.  The C<into> and C<into_level>
+options change where those exported routines are installed.
+
+  generator  - a callback used to produce the code that will be installed
+               default: Sub::Exporter::default_generator
+
+  installer  - a callback used to install the code produced by the generator
+               default: Sub::Exporter::default_installer
+
+For information on how these callbacks are used, see the documentation for
+C<L</default_generator>> and C<L</default_installer>>.
+
 =head2 Export Configuration
 
 The C<exports> list may be provided as an array reference or a hash reference.
@@ -365,11 +383,13 @@ first argument, in a hashref.
 
 These options are:
 
-  into_level - how far up the caller stack to look for a target (default 0)
-  into       - an explicit target (package) into which to export routines
+  into_level
+  into
+  generator
+  installer
 
-Providing both C<into_level> and C<into> will cause an exception to be
-thrown.
+These override the same-named configuration options described in L</EXPORTER
+CONFIGURATION>.
 
 =cut
 
@@ -504,8 +524,6 @@ sub _collect_collections {
     Carp::croak "collection $name provided multiple times in import"
       if $seen{ $name }++;
 
-    $collection{ $name } = $value;
-
     if (ref(my $hook = $config->{collectors}{$name})) {
       my $arg = {
         name        => $name,
@@ -522,6 +540,8 @@ sub _collect_collections {
         Carp::croak $error_msg unless $hook->($value, $arg);
       }
     }
+
+    $collection{ $name } = $value;
   }
 
   return \%collection;
@@ -552,9 +572,6 @@ The exporter is built by C<L</build_exporter>>.
 
 =cut
 
-# \%special is for experimental options that may or may not be kept around and,
-# probably, moved to \%config.  These are also passed along to build_exporter.
-
 sub setup_exporter {
   my ($config)  = @_;
 
@@ -601,7 +618,8 @@ my %valid_config_key;
 BEGIN {
   %valid_config_key =
     map { $_ => 1 }
-    qw(collectors exporter exports groups into into_level)
+    qw(collectors installer generator exports groups into into_level),
+    qw(exporter), # deprecated
 }
 
 sub _rewrite_build_config {
@@ -611,6 +629,15 @@ sub _rewrite_build_config {
     Carp::croak "unknown options (@keys) passed to Sub::Exporter";
   }
 
+  Carp::croak q(into and into_level may not both be supplied to exporter)
+    if exists $config->{into} and exists $config->{into_level};
+
+  # XXX: Remove after deprecation period.
+  if ($config->{exporter}) {
+    Carp::cluck "'exporter' argument to build_exporter is deprecated. Use 'installer' instead; the semantics are identical.";
+    $config->{installer} = delete $config->{exporter};
+  }
+
   Carp::croak q(into and into_level may not both be supplied to exporter)
     if exists $config->{into} and exists $config->{into_level};
 
@@ -642,6 +669,9 @@ sub _rewrite_build_config {
 
   # by default, build an all-inclusive 'all' group
   $config->{groups}{all} ||= [ keys %{ $config->{exports} } ];
+
+  $config->{generator} ||= \&default_generator;
+  $config->{installer} ||= \&default_installer;
 }
 
 sub build_exporter {
@@ -657,6 +687,11 @@ sub build_exporter {
     Carp::croak q(into and into_level may not both be supplied to exporter)
       if exists $special->{into} and exists $special->{into_level};
 
+    if ($special->{exporter}) {
+      Carp::cluck "'exporter' special import argument is deprecated. Use 'installer' instead; the semantics are identical.";
+      $special->{installer} = delete $special->{exporter};
+    }
+
     my $into
       = defined $special->{into}       ? delete $special->{into}
       : defined $special->{into_level} ? caller(delete $special->{into_level})
@@ -664,9 +699,8 @@ sub build_exporter {
       : defined $config->{into_level}  ? caller($config->{into_level})
       :                                  caller(0);
 
-    my $export = delete $special->{exporter}
-              || $config->{exporter}
-              || \&default_exporter;
+    my $generator = delete $special->{generator} || $config->{generator};
+    my $installer = delete $special->{installer} || $config->{installer};
 
     # this builds a AOA, where the inner arrays are [ name => value_ref ]
     my $import_args = Data::OptList::mkopt([ @_ ]);
@@ -679,61 +713,67 @@ sub build_exporter {
     my $to_import = _expand_groups($class, $config, $import_args, $collection);
 
     # now, finally $import_arg is really the "to do" list
-    for (@$to_import) {
-      _do_import($class, @$_, $collection, $config, $into, $export);
-    }
+    _do_import(
+      {
+        class     => $class,
+        col       => $collection,
+        config    => $config,
+        into      => $into,
+        generator => $generator,
+        installer => $installer,
+      },
+      $to_import,
+    );
   };
 
   return $import;
 }
 
 sub _do_import {
-  my ($class, $name, $arg, $collection, $config, $into, $export) = @_;
+  my ($arg, $to_import) = @_;
 
-  my ($generator, $as);
+  my @todo;
 
-  if ($arg and Params::Util::_CODELIKE($arg)) { ## no critic
-    # This is the case when a group generator has inserted name/code pairs.
-    $generator = sub { $arg };
-    $as = $name;
-  } else {
-    $arg = { $arg ? %$arg : () };
+  for my $pair (@$to_import) {
+    my ($name, $import_arg) = @$pair;
 
-    Carp::croak qq("$name" is not exported by the $class module)
-      unless (exists $config->{exports}{$name});
+    my ($generator, $as);
 
-    $generator = $config->{exports}{$name};
+    if ($import_arg and Params::Util::_CODELIKE($import_arg)) { ## no critic
+      # This is the case when a group generator has inserted name/code pairs.
+      $generator = sub { $import_arg };
+      $as = $name;
+    } else {
+      $import_arg = { $import_arg ? %$import_arg : () };
 
-    $as = exists $arg->{-as} ? (delete $arg->{-as}) : $name;
+      Carp::croak qq("$name" is not exported by the $arg->{class} module)
+        unless exists $arg->{config}{exports}{$name};
+
+      $generator = $arg->{config}{exports}{$name};
+
+      $as = exists $import_arg->{-as} ? (delete $import_arg->{-as}) : $name;
+    }
+
+    my $code = $arg->{generator}->(
+      { 
+        class     => $arg->{class},
+        name      => $name,
+        arg       => $import_arg,
+        col       => $arg->{col},
+        generator => $generator,
+      }
+    );
+
+    push @todo, $as, $code;
   }
 
-  $export->($class, $generator, $name, $arg, $collection, $as, $into);
-}
-
-# XXX: Consider implementing a _export_args routine that takes the arguments to
-# _export and returns a hash of named params.  This lets other people write
-# exporters without tying me down to one set of @_ contents.  Maybe that's
-# premature guarantee, though, unless I guarantee that @_ will never get
-# /smaller/.
-
-=head2 default_exporter
-
-This is Sub::Exporter's default exporter.  It does what Sub::Exporter promises:
-it calls generators with the three normal arguments, then installs the code
-into the target package.
-
-B<Warning!>  Its interface isn't really stable yet, so don't rely on it.  It's
-only named here so that you can pass it in to the exporter builder.  It will
-have a stable interface in the future so that it may be more easily replaced.
-
-=cut
-
-sub default_exporter {
-  my ($class, $generator, $name, $arg, $collection, $as, $into) = @_;
-  _install(
-    _generate($class, $generator, $name, $arg, $collection),
-    $into,
-    $as,
+  $arg->{installer}->(
+    {
+      class => $arg->{class},
+      into  => $arg->{into},
+      col   => $arg->{col},
+    },
+    \@todo,
   );
 }
 
@@ -752,8 +792,28 @@ sub default_exporter {
 #   }
 # }
 
-sub _generate {
-  my ($class, $generator, $name, $arg, $collection) = @_;
+=head2 default_generator
+
+This is Sub::Exporter's default generator.  It takes bits of configuration that
+have been gathered during the import and turns them into a coderef that can be
+installed.
+
+  my $code = default_generator(\%arg);
+
+Passed arguments are:
+
+  class - the class on which the import method was called
+  name  - the name of the export being generated
+  arg   - the arguments to the generator
+  col   - the collections
+
+  generator - the generator to be used to build the export (code or scalar ref)
+
+=cut
+
+sub default_generator {
+  my ($arg) = @_;
+  my ($class, $name, $generator) = @$arg{qw(class name generator)};
 
   if (not defined $generator) {
     my $code = $class->can($name)
@@ -764,28 +824,60 @@ sub _generate {
   # I considered making this "$class->$generator(" but it seems that
   # overloading precedence would turn an overloaded-as-code generator object
   # into a string before code. -- rjbs, 2006-06-11
-  return $generator->($class, $name, $arg, $collection)
+  return $generator->($class, $name, $arg->{arg}, $arg->{col})
     if Params::Util::_CODELIKE($generator); ## no critic Private
 
   # This "must" be a scalar reference, to a generator method name.
   # -- rjbs, 2006-12-05
-  return $class->$$generator($name, $arg, $collection);
+  return $class->$$generator($name, $arg->{arg}, $arg->{col});
 }
 
-sub _install {
-  my ($code, $into, $as) = @_;
-  # Allow as isa ARRAY to push onto an array?
-  # Allow into isa HASH to install name=>code into hash?
+=head2 default_installer
 
-  if (ref $as eq 'SCALAR') {
-    $$as = $code;
-  } elsif (ref $as) {
-    Carp::croak "invalid reference type for $as: " . ref $as;
-  } else {
-    Sub::Install::reinstall_sub({ code => $code, into => $into, as => $as });
+This is Sub::Exporter's default installer.  It does what Sub::Exporter
+promises: it installs code into the target package.
+
+  default_installer(\%arg, \@to_export);
+
+Passed arguments are:
+
+  into - the package into which exports should be delivered
+
+C<@to_export> is a list of name/value pairs.  The default exporter assigns code
+(the values) to named slots (the names) in the given package.  If the name is a
+scalar reference, the scalar reference is made to point to the code reference
+instead.
+
+=cut
+
+sub default_installer {
+  my ($arg, $to_export) = @_;
+
+  for (my $i = 0; $i < @$to_export; $i += 2) {
+    my ($as, $code) = @$to_export[ $i, $i+1 ];
+
+    # Allow as isa ARRAY to push onto an array?
+    # Allow into isa HASH to install name=>code into hash?
+
+    if (ref $as eq 'SCALAR') {
+      $$as = $code;
+    } elsif (ref $as) {
+      Carp::croak "invalid reference type for $as: " . ref $as;
+    } else {
+      Sub::Install::reinstall_sub({
+        code => $code,
+        into => $arg->{into},
+        as   => $as
+      });
+    }
   }
 }
 
+sub default_exporter {
+  Carp::cluck "default_exporter is deprecated; call default_installer instead; the semantics are identical";
+  goto &default_installer;
+}
+
 =head1 EXPORTS
 
 Sub::Exporter also offers its own exports: the C<setup_exporter> and
