@@ -12,13 +12,11 @@ Sub::Exporter::Util - utilities to make Sub::Exporter easier
 
 =head1 VERSION
 
-version 0.976
-
-  $Id$
+version 0.978
 
 =cut
 
-our $VERSION = '0.976';
+our $VERSION = '0.978';
 
 =head1 DESCRIPTION
 
@@ -226,15 +224,15 @@ sub merge_col {
   return %merged;
 }
 
-=head2 mixin_exporter
+=head2 mixin_installer
 
   use Sub::Exporter -setup => {
-    exporter => Sub::Exporter::Util::mixin_exporter,
-    exports  => [ qw(foo bar baz) ],
+    installer => Sub::Exporter::Util::mixin_installer,
+    exports   => [ qw(foo bar baz) ],
   };
 
-This utility returns an exporter that will export into a superclass and adjust
-the ISA importing class to include the newly generated superclass.
+This utility returns an installer that will install into a superclass and
+adjust the ISA importing class to include the newly generated superclass.
 
 If the target of importing is an object, the hierarchy is reversed: the new
 class will be ISA the object's class, and the object will be reblessed.
@@ -260,26 +258,25 @@ sub __mixin_class_for {
   return $mixin_class;
 }
 
-sub mixin_exporter {
-  # These are NOT arguments to mixin_exporter, that's why there is no = @_.
-  # They are variables created to enclose in each generated exporter coderef.
-  my ($mixin_class, $col_ref);
-
+sub mixin_installer {
   sub {
-    my ($class, $generator, $name, $arg, $collection, $as, $into) = @_;
+    my ($arg, $to_export) = @_;
 
-    unless ($mixin_class and ($collection == $col_ref)) {
-      $mixin_class = __mixin_class_for($class, $into);
-      bless $into => $mixin_class if ref $into;
-      $col_ref = 0 + $collection;
-    }
+    my $mixin_class = __mixin_class_for($arg->{class}, $arg->{into});
+    bless $arg->{into} => $mixin_class if ref $arg->{into};
 
-    Sub::Exporter::default_exporter(
-      $class, $generator, $name, $arg, $collection, $as, $mixin_class
+    Sub::Exporter::default_installer(
+      { %$arg, into => $mixin_class },
+      $to_export,
     );
   };
 }
 
+sub mixin_exporter {
+  Carp::cluck "mixin_exporter is deprecated; use mixin_installer instead; it behaves identically";
+  return mixin_installer;
+}
+
 =head2 like
 
 It's a collector that adds imports for anything like given regex.
@@ -339,7 +336,7 @@ use Sub::Exporter -setup => {
     merge_col
     curry_method curry_class
     curry_chain
-    mixin_exporter
+    mixin_installer mixin_exporter
   ) ]
 };
 
